contentstream

package
v1.3.0 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Nov 29, 2025 License: MIT Imports: 4 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

This section is empty.

Types

type Operation

type Operation struct {
	Operator string        // The operator (e.g., "Tj", "Tm", "q")
	Operands []core.Object // The operands
}

Operation represents a single content stream operation consisting of an operator and its operands

type Parser

type Parser struct {
	// contains filtered or unexported fields
}

Parser parses PDF content streams

func NewParser

func NewParser(data []byte) *Parser

NewParser creates a new content stream parser

func (*Parser) Parse

func (p *Parser) Parse() ([]Operation, error)

Parse parses the content stream and returns a list of operations

Jump to