Documentation
¶
Overview ¶
Package format provides file format detection for the tabula library.
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type Format ¶
type Format int
Format represents a supported document format.
func DetectFromMagic ¶
DetectFromMagic checks file magic bytes to determine format. This provides more reliable detection than extension-based detection. Returns Unknown if the format cannot be determined from magic bytes alone.
func DetectFromReader ¶
DetectFromReader inspects the content to determine format. This is more reliable than extension-based detection and can distinguish between different ZIP-based formats (DOCX, XLSX, PPTX).
Click to show internal directories.
Click to hide internal directories.